@@ -6,7 +6,7 @@ from cubicweb import Binary
 options = (
     ('s3-auto-delete', {
         'type': 'yn',
-        'default': False,
+        'default': True,
         'help': 'Delete S3 objects on entity deletion',
         'group': 's3',
         'level': 2}),
