@@ -1,4 +1,4 @@
-# copyright 2018 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2018-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of CubicWeb.
@@ -28,7 +28,7 @@ import boto3
 from cubicweb import Binary, set_log_methods
 from cubicweb.server.sources.storages import Storage
 from cubicweb.server.edition import EditedEntity
-from cubicweb.server.hook import DataOperationMixIn, Operation
+from cubicweb.server.hook import DataOperationMixIn, LateOperation
 
 
 class S3Storage(Storage):
@@ -51,12 +51,18 @@ class S3Storage(Storage):
         """see docstring for prototype, which vary according to is_source_callback
         """
         key = source.binary_to_str(value).decode('utf-8')
+        if cnx.commit_state == 'precommit':
+            # download suffixed key if it exists
+            # FIXME need a way to check that the attribute is actually edited
+            try:
+                suffixed_key = self.suffixed_key(key)
+                return self.download(suffixed_key)
+            except Exception:
+                pass
         try:
-            result = self.s3cnx.get_object(Bucket=self.bucket, Key=key)
-            self.info('Downloaded %s/%s from S3', self.bucket, key)
-            return Binary(result['Body'].read())
+            return self.download(key)
         except Exception as ex:
-            source.critical("can't retrive S3 object %s: %s", value, ex)
+            source.critical("can't retrieve S3 object %s: %s", value, ex)
             return None
 
     def entity_added(self, entity, attr):
@@ -68,8 +74,7 @@ class S3Storage(Storage):
             if PY3:
                 key = key.decode('utf-8')
             try:
-                return Binary(self.s3cnx.get_object(
-                    Bucket=self.bucket, Key=key)['Body'].read())
+                return self.download(key)
             except Exception:
                 return None
 
@@ -93,7 +98,9 @@ class S3Storage(Storage):
             buffer = Binary(binary.read())
             binary.seek(0)
             suffixed_key = self.suffixed_key(key)
-            self.s3cnx.upload_fileobj(buffer, self.bucket, suffixed_key)
+            extra_args = self.get_upload_extra_args(entity, attr, key)
+            self.s3cnx.upload_fileobj(buffer, self.bucket, suffixed_key,
+                                      ExtraArgs=extra_args)
             buffer.close()
 
             # move to final key in post commit event
@@ -103,6 +110,13 @@ class S3Storage(Storage):
                       entity.eid, attr, self.bucket, suffixed_key)
         return binary
 
+    def get_upload_extra_args(self, _entity, _attr, _key):
+        """Additional options for boto3's upload_fileobj method.
+        Documentation for supported options can be found at:
+        https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-uploading-files.html#the-extraargs-parameter
+        """
+        return {}
+
     def entity_updated(self, entity, attr):
         """an entity using this storage for attr has been updatded"""
         return self.entity_added(entity, attr)
@@ -137,10 +151,6 @@ class S3Storage(Storage):
         """
         Return the S3 key of the S3 object storing the content of attribute
         attr of the entity.
-
-        If the given entity already has a key (eg. at entity creation time),
-        a new  key is generated.
-
         """
         try:
             rset = entity._cw.execute(
@@ -160,18 +170,27 @@ class S3Storage(Storage):
     def new_s3_key(self, entity, attr):
         """Generate a new key for given entity attr.
 
-        This implemenation just return a random UUID"""
+        This implementation just returns a random UUID"""
         return str(uuid.uuid1())
 
     def suffixed_key(self, key):
         return key + self.suffix
 
+    def download(self, key):
+        result = self.s3cnx.get_object(Bucket=self.bucket, Key=key)
+        self.info('Downloaded %s/%s from S3', self.bucket, key)
+        return Binary(result['Body'].read())
 
-class S3AddFileOp(DataOperationMixIn, Operation):
+
+class S3AddFileOp(DataOperationMixIn, LateOperation):
     containercls = list
 
     def postcommit_event(self):
+        consumed_keys = set()
         for storage, key, eid, attr in self.get_data():
+            if key in consumed_keys:
+                continue
+            consumed_keys.add(key)
             suffixed_key = storage.suffixed_key(key)
             storage.s3cnx.copy_object(
                 Bucket=storage.bucket,
@@ -192,7 +211,7 @@ class S3AddFileOp(DataOperationMixIn, Operation):
                       eid, attr, storage.bucket, suffixed_key)
 
 
-class S3DeleteFileOp(DataOperationMixIn, Operation):
+class S3DeleteFileOp(DataOperationMixIn, LateOperation):
     containercls = list
 
     def postcommit_event(self):
