@@ -10,7 +10,7 @@ class S3StorageTestMixin(object):
     def setUp(self):
         s3_mock = mock_s3()
         s3_mock.start()
-        resource = boto3.resource('s3', region_name='somewhere')
+        resource = boto3.resource('s3', region_name='us-east-1')
         self.s3_bucket = resource.create_bucket(Bucket=self.s3_bucket)
         patched_storage_s3_client = patch(
             'cubicweb_s3storage.storages.S3Storage._s3_client',
